"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getTableName = exports.getFieldMetadata = exports.setObjectMetadata = exports.getObjectMetadata = exports.initFieldMetadata = exports.initObjectMetadata = void 0;
const constants_1 = require("./constants");
const DataType_1 = require("./DataType");
const Errors_1 = require("../system/Errors");
const getReflectedDataType = (prototype, propertyKey) => {
    let t = Reflect.getMetadata("design:type", prototype, propertyKey);
    if (!t) {
        throw Errors_1.Errors.SYSTEM_ERROR(`Cannot get reflected data type for ${prototype.constructor.name}:${propertyKey}`);
    }
    switch (t.name) {
        case "GeoPoint":
            return DataType_1.DataType.POINT;
        case "Boolean":
            return DataType_1.DataType.BOOLEAN;
        case "Number":
            return DataType_1.DataType.NUMBER;
        case "String":
            return DataType_1.DataType.STRING;
        case "Array":
            return DataType_1.DataType.ARRAY;
        default:
            return DataType_1.DataType.OBJECT;
    }
};
const initObjectMetadata = (prototype) => {
    if (prototype === Object.prototype) {
        return {};
    }
    if (!prototype.hasOwnProperty(constants_1.METADATA)) {
        const superPrototype = (0, exports.initObjectMetadata)(Object.getPrototypeOf(prototype));
        prototype[constants_1.METADATA] = Object.assign(Object.assign({}, superPrototype), { id: (0, exports.getTableName)(prototype.constructor), fields: superPrototype.fields ? superPrototype.fields.map((f) => (Object.assign({}, f))) : [] });
    }
    return prototype[constants_1.METADATA];
};
exports.initObjectMetadata = initObjectMetadata;
const initFieldMetadata = (prototype, propertyKey) => {
    let meta = (0, exports.initObjectMetadata)(prototype);
    let fieldIndex = meta.fields.findIndex((f) => f.name === propertyKey);
    if (fieldIndex < 0) {
        const field = {
            name: propertyKey,
            dataType: getReflectedDataType(prototype, propertyKey),
            description: "",
        };
        meta.fields.push(field);
        return field;
    }
    else {
        //-- shadow clone the field
        const field = Object.assign({}, meta.fields[fieldIndex]);
        meta.fields[fieldIndex] = field;
        return field;
    }
};
exports.initFieldMetadata = initFieldMetadata;
const getObjectMetadata = (obj) => obj.prototype[constants_1.METADATA];
exports.getObjectMetadata = getObjectMetadata;
const setObjectMetadata = (obj, metadata) => {
    obj.prototype[constants_1.METADATA] = metadata;
};
exports.setObjectMetadata = setObjectMetadata;
const getFieldMetadata = (cls, key) => {
    const clsMeta = (0, exports.getObjectMetadata)(cls);
    return clsMeta === null || clsMeta === void 0 ? void 0 : clsMeta.fields.find((f) => f.name === key);
};
exports.getFieldMetadata = getFieldMetadata;
const getTableName = (model) => model === null || model === void 0 ? void 0 : model.name.toLowerCase();
exports.getTableName = getTableName;
