"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CachedDataContainer = void 0;
const request_serializer_1 = require("./request-serializer");
class CachedDataContainer {
    constructor(valueGetterFn, valueSetterFn, dataRetentionDurationMs = 30000) {
        this.valueGetterFn = valueGetterFn;
        this.valueSetterFn = valueSetterFn;
        this.dataRetentionDurationMs = dataRetentionDurationMs;
        this.cache = {};
        this.valueProvider = new request_serializer_1.SerializedValueProvider(this.valueGetterFn);
    }
    getValue(...args) {
        return __awaiter(this, void 0, void 0, function* () {
            const key = args.map((arg) => String(arg)).join(":");
            let cache = this.cache[key];
            if (cache === undefined) {
                cache = {
                    lastFetchTimestamp: 0,
                };
                this.cache[key] = cache;
            }
            if (cache.lastFetchTimestamp + this.dataRetentionDurationMs < Date.now()) {
                const value = yield this.valueProvider.getValue(...args);
                cache.value = value;
                cache.lastFetchTimestamp = Date.now();
            }
            return cache.value;
        });
    }
    setValue(value, ...args) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const key = args.map((arg) => String(arg)).join(":");
            let cache = this.cache[key];
            if (cache === undefined) {
                cache = {
                    lastFetchTimestamp: Date.now(),
                };
                this.cache[key] = cache;
            }
            cache.value = value;
            yield ((_a = this.valueSetterFn) === null || _a === void 0 ? void 0 : _a.call(this, value, ...args));
        });
    }
}
exports.CachedDataContainer = CachedDataContainer;
