"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SerializedValueProvider = void 0;
class SerializedValueProvider {
    constructor(fn) {
        this.fn = fn;
        this.resolvers = {};
    }
    getValue(...args) {
        return __awaiter(this, void 0, void 0, function* () {
            const key = args.map((arg) => String(arg)).join(":");
            let resolvers = this.resolvers[key];
            if (!resolvers) {
                resolvers = {
                    fetching: false,
                    resolvers: [],
                };
                this.resolvers[key] = resolvers;
            }
            if (resolvers.fetching) {
                //-- let's queue here
                return new Promise((resolve) => {
                    resolvers.resolvers.push(resolve);
                });
            }
            //-- not fetching, let's do it
            resolvers.fetching = true;
            const value = yield this.fn(...args);
            //-- notify those waiting
            for (const resolveFn of resolvers.resolvers) {
                resolveFn(value);
            }
            resolvers.fetching = false;
            resolvers.resolvers = [];
            return value;
        });
    }
}
exports.SerializedValueProvider = SerializedValueProvider;
