"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Injectable = exports.Inject = exports.Provider = exports.Register = void 0;
const metadata_1 = require("../common/metadata");
const injection_1 = require("./injection");
const Errors_1 = require("../system/Errors");
const Register = (config) => (target) => {
    const serviceProvider = (0, injection_1.getServiceProvider)();
    serviceProvider.register(target, (config === null || config === void 0 ? void 0 : config.mode) || "singleton");
};
exports.Register = Register;
const Provider = (baseClass, mode) => (constructor) => {
    (0, exports.Register)({ mode: "singleton" })(constructor);
    const metadata = (0, metadata_1.initObjectMetadata)(constructor.prototype);
    metadata.injectionMode = mode || "singleton";
    const serviceProvider = (0, injection_1.getServiceProvider)();
    serviceProvider.addProvider(baseClass, constructor);
};
exports.Provider = Provider;
const Inject = (baseClass, required = true) => ((target, propertyKey, index) => {
    if (propertyKey !== undefined) {
        throw Errors_1.ErrorBuilder.forInvalidInjection(`@Inject decorator used outside constructor method ${target.name}:${propertyKey}:${index}`);
    }
    let metadata = (0, metadata_1.initObjectMetadata)(target.prototype);
    if (!metadata.constructorDependencies) {
        metadata.constructorDependencies = (Reflect.getMetadata("design:paramtypes", target) || []).map((ctor) => ({ ctor, required: true }));
    }
    if (metadata.constructorDependencies) {
        metadata.constructorDependencies[index].ctor = baseClass;
        metadata.constructorDependencies[index].required = required;
    }
});
exports.Inject = Inject;
const Injectable = () => (_target) => {
    //-- this is only being used to trigger parameter metadata in constructor
};
exports.Injectable = Injectable;
