"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ErrorBuilder = exports.Errors = exports._ = void 0;
const _ = (name, code) => (message) => ({ name, code, message });
exports._ = _;
exports.Errors = {
    //-- server errors
    SYSTEM_ERROR: (0, exports._)("SYSTEM_ERROR", 500),
    QUERY_ERROR: (0, exports._)("QUERY_ERROR", 500),
    CANNOT_LOCK: (0, exports._)("CANNOT_LOCK", 500),
    //-- client errors
    VALIDATION_ERROR: (0, exports._)("VALIDATION_ERROR", 400),
    ACCESS_DENIED: (0, exports._)("ACCESS_DENIED", 403),
    TFA_REQUIRED: (0, exports._)("TFA_REQUIRED", 403),
    SESSION_EXPIRED: (0, exports._)("SESSION_EXPIRED", 403),
    NOT_FOUND: (0, exports._)("NOT_FOUND", 404),
    HTTP_REQUEST_ERROR: (0, exports._)("HTTP_REQUEST_ERROR", 405),
    BAD_STATE: (0, exports._)("BAD_STATE", 422),
    TOKEN_EXPIRED: (0, exports._)("TOKEN_EXPIRED", 422),
    INVALID_TOKEN: (0, exports._)("INVALID_TOKEN", 422),
    QUERY_UNIQUE_CONSTRAINT: (0, exports._)("QUERY_UNIQUE_CONSTRAINT", 422),
    QUERY_CASCADE_RESTRICTION: (0, exports._)("QUERY_CASCADE_RESTRICTION", 422),
    INVALID_CREDENTIALS: (0, exports._)("INVALID_CREDENTIALS", 422),
    REQUEST_RATE_LIMIT: (0, exports._)("REQUEST_RATE_LIMIT", 429),
};
class ErrorBuilder {
    static forConfigNotFound(configName) {
        return exports.Errors.SYSTEM_ERROR(`Config not set for ${configName}`);
    }
    static forTransientUsedWithInitable(constructorFn) {
        return exports.Errors.SYSTEM_ERROR(`Initable cannot be used with transient, found in ${constructorFn.name}`);
    }
    static forInvalidInjection(message) {
        return exports.Errors.SYSTEM_ERROR(message);
    }
    static forInjectionCircularDependency(constructorFns) {
        return exports.Errors.SYSTEM_ERROR(`Circular dependency injection detected between: ${constructorFns.reduce((list, constructor) => `${list}${constructor.name};`, "")}`);
    }
    static forInjectionNotFound(constructorFn) {
        return exports.Errors.SYSTEM_ERROR(`Injection not found for: ${constructorFn.name}`);
    }
    static forMutipleInjectionsFound(constructorFn, candidates) {
        return exports.Errors.SYSTEM_ERROR(`Mutiple injection candidate found for: ${constructorFn.name}, namely: ${candidates.reduce((list, candidate) => `${list}${candidate.name};`, "")}`);
    }
    static forMutipleProvidersFound(constructorFn, candidates) {
        return exports.Errors.SYSTEM_ERROR(`Mutiple injection candidate found for: ${constructorFn.name}, namely: ${candidates.reduce((list, candidate) => `${list}${candidate.name};`, "")}`);
    }
}
exports.ErrorBuilder = ErrorBuilder;
